/*
 *  Accelerator.java
 *	Part of the Spirograph problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's Computers
 * and Cognition Group. For more information, see 
 * <a href="http://www.cognition.olin.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@olin.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Copyright (C) 2003 Franklin W. Olin College of Engineering.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

/** This class has probably drifted from it's intent, and might be better named
 * Mover. Objects implementing this interface can receive position and velocity
 * information and generate a <code>double</code>. This value is usually
 * inserted into the next iteration of a motion equations.
 *
 * <P>Copyright (C) 1998 Massachusetts Institute of Technology.<br />
 * Copyright &copy; 2002-2003 Franklin W. Olin College of Engineering.</p>
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @author Patrick G Heck, gus.heck@olin.edu
 * @version $Id: Accelerator.java,v 1.3 2004/02/09 20:55:03 gus Exp $
 */
public interface Accelerator {

    /**
     * Calculate a new motion related value from position and velocity information.
     * Usually this value represents, position, acceleration, or velocity.
     *
     * @param pos Current position on this axis
     * @param vel Current velocity on this axis
     * @param otherPos Current position on the other axis
     * @param otherVel Current Velocity on the other axis
     * @param maxPos The maximum position on this axis (+ or -)
     * @return The new value
     */    
    public double act(double pos, double vel,
		      double otherPos, double otherVel, double maxPos);
    
}

/*
 * $Log: Accelerator.java,v $
 * Revision 1.3  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.2  2003/01/10 22:19:18  gus
 * Complete and update the javadocs
 *
 */