/*
 *  CodeMerger.java
 *	Part of the Spirograph problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.awt.event.*;
import java.awt.*;

/**
 * This class implements a listener that is added to the "Load
 * Horizontal Code." and "Load Vertical Code" buttons on the Code
 * Box. The CodeMerger reads the text from the text fields and
 * arranges for the creation of an Accelerator object that evaluates
 * the code. This is then passed to the AccelHandlers as a new
 * Accelerator to use. <p>
 *
 *  <p>Copyright &copy; 1998 Massachusetts Institute of Technology.<br />
 * Copyright &copy; 2002-2003 Franklin W. Olin College of Engineering.</p>
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @author Nathan Williams, nathanw@mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: CodeMerger.java,v 1.5 2004/02/09 20:55:03 gus Exp $
 * @see DotPanel
 */


public class CodeMerger implements ActionListener {

  private String ruleName; 
    
  private AccelHandler myAccelHandler, otherAccelHandler;
  private boolean useAsBoth = false;
  private CodeSource studentInput;

  //Loads the compiled classes.  Must be used to load classes that have
  // changed after they have been loaded once.
  // private MultiClassLoader multiClassLoader;
  
  /** Create a code merger that can convert student input to working behavior
   * and give it a name.
   * @param ruleName The name for this code merger
   * @param studentInput The code the student wrote that needs to be merged.
   * @param myAccelHandler The object that will contain the behavior for this axis
   * @param otherAccelHandler The object that contians the behavior for the other axis
   */
  
  public CodeMerger(String ruleName, CodeSource studentInput, AccelHandler myAccelHandler, AccelHandler otherAccelHandler) {
    this.ruleName = ruleName;  // keeping this because it makes code more
                               // readable. It is otherwise unused.
    this.studentInput = studentInput;
    this.myAccelHandler = myAccelHandler;
    this.otherAccelHandler = otherAccelHandler;
  }

  /** This is used when the option to use the same rule for both the
   * vertical and horizontal acceleration is selected
   * @param useAsBoth Is this rule used for both directions?
   */
  public void setBoth(boolean useAsBoth) {
    this.useAsBoth = useAsBoth;
  }

  /** When the event we're registered for is performed, create the
   * Accelerator from the code.
   * @param ae An event signaling that the code should be merged
   */
  public void actionPerformed(ActionEvent ae) {
    BshAccel b;

    b = new BshAccel(studentInput.getFields(), studentInput.getCode());

    this.myAccelHandler.setAccel((Accelerator) b);

    System.out.println ("Use as both rules: " + useAsBoth);
    if (useAsBoth) {
      this.otherAccelHandler.setAccel((Accelerator) b);
    }
    System.out.println ("Done loading code.");
  }
}

/*
 * $Log: CodeMerger.java,v $
 * Revision 1.5  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.4  2003/01/13 19:59:25  gus
 * Remove entirely unneccessary argument from BshAccel constructor
 *
 * Revision 1.3  2003/01/10 22:19:18  gus
 * Complete and update the javadocs
 *
 */
