/*
 * SpiroControl.java
 *
 * Created on December 5, 2002, 12:02 PM
 */

package spirograph;

import java.awt.Frame;
import java.util.Vector;
import javax.swing.JTabbedPane;
import java.awt.event.WindowEvent;
import javax.swing.event.ChangeEvent;

/** This is a replacement class for the current main class of the
 * Etch-a-sketch (aka apirograph). This implementation intends
 * to ensure that the entire application fits on a single screen,
 * and operates with a clean, intuitive interface. By the time this
 * problem set is fully reworked it should implement a MVC architecture.
 *
 * <p>Copyright &copy; 2002-2003 Franklin W. Olin College of Engineering.</p>
 *
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: EtchControl.java,v 1.7 2003/01/15 18:26:50 gus Exp $
 */
public class EtchControl extends javax.swing.JFrame {
    
    private static String xArg;
    private static String yArg;
    private static Thread xSpirit;
    private static Thread ySpirit;
    private static Accelerator xAccel;
    private static Accelerator yAccel;
    private static AccelHandler xHandler;
    private static AccelHandler yHandler;
    private static Coord xCoord = new Coord();
    private static Coord yCoord = new Coord();
    
    
    private boolean ready2Go = false;
    private boolean xStarted, yStarted;
    private boolean running = false;

    private DotFrame myFrame;
    
    // our initial recipies
    private Recipie vert = new Recipie();
    private Recipie horz = new Recipie();
    
    // these are our code mergers that do the work of translating studen
    // input into useable code at the click of a button 
    private CodeMerger vMerger = new CodeMerger("Vertical",
                                                vert,yHandler,xHandler);
    private CodeMerger hMerger = new CodeMerger("Horizontal",
                                                horz,xHandler, yHandler);
    

    // These are for notifying any listening EtchControlListeners
    private Vector listeners = new Vector();
    private ChangeEvent ce = new ChangeEvent(this);

    
    /** Creates new form SpiroControl. There should only ever be a single 
     *  <code>EtchControl</code> an thus this constructor is private.
     */
    private EtchControl() {
        
        initComponents();
        
        recipieViewVert.setRecipie(vert);
        addEtchControlListener(recipieViewVert);
        
        // Set the title on the tab to match the recipie's id num
        ((JTabbedPane) recipieViewVert.getParent()).setTitleAt(
         ((JTabbedPane) recipieViewVert.getParent()).
            indexOfComponent(recipieViewVert), 
         "Recipie " + recipieViewVert.getRecipieNum());

        recipieViewHorz.setRecipie(horz);
        addEtchControlListener(recipieViewHorz);
        
        // Set the title on the tab to match the recipie's id num
        ((JTabbedPane) recipieViewHorz.getParent()).setTitleAt(
         ((JTabbedPane) recipieViewHorz.getParent()).
            indexOfComponent(recipieViewHorz), 
         "Recipie " + recipieViewHorz.getRecipieNum());
        
        updateListeners();
        ready2Go = true;
        recipieTabsStateChanged(ce);
        
        myFrame = new DotFrame(xCoord, yCoord);

    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        titleLabelPanel = new javax.swing.JPanel();
        titleLabel = new javax.swing.JLabel();
        centerPanel = new javax.swing.JPanel();
        recipieTabs = new javax.swing.JTabbedPane();
        recipieViewVert = new spirograph.RecipieView();
        recipieViewHorz = new spirograph.RecipieView();
        ControlButtonPanel = new javax.swing.JPanel();
        startButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        resetButton = new javax.swing.JButton();
        newRecipieButton = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        newRecipie = new javax.swing.JMenuItem();
        advOptionsMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        exitMenuItem = new javax.swing.JMenuItem();
        recipieMenu = new javax.swing.JMenu();
        useVertCheckBoxMenuItem = new javax.swing.JCheckBoxMenuItem();
        useHorzCheckBoxMenuItem = new javax.swing.JCheckBoxMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        compileRecipieMenuItem = new javax.swing.JMenuItem();
        compileBothRecipieMenuItem = new javax.swing.JMenuItem();
        standaloneRecipieMenuItem = new javax.swing.JMenuItem();

        setTitle("Etch-a-Sketch");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        titleLabelPanel.setMinimumSize(new java.awt.Dimension(0, 0));
        titleLabelPanel.setPreferredSize(new java.awt.Dimension(100, 25));
        titleLabel.setFont(new java.awt.Font("Dialog", 1, 18));
        titleLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        titleLabel.setText("Etch-a-Sketch Controls");
        titleLabelPanel.add(titleLabel);

        getContentPane().add(titleLabelPanel, java.awt.BorderLayout.NORTH);

        centerPanel.setLayout(new java.awt.GridBagLayout());

        centerPanel.setPreferredSize(new java.awt.Dimension(801, 350));
        recipieTabs.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                recipieTabsStateChanged(evt);
            }
        });

        recipieViewVert.setBorder(null);
        recipieTabs.addTab("tab4", recipieViewVert);

        recipieTabs.addTab("tab5", recipieViewHorz);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        centerPanel.add(recipieTabs, gridBagConstraints);

        getContentPane().add(centerPanel, java.awt.BorderLayout.CENTER);

        ControlButtonPanel.setLayout(new java.awt.GridLayout(2, 0));

        startButton.setText("Start");
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButtonActionPerformed(evt);
            }
        });

        ControlButtonPanel.add(startButton);

        stopButton.setText("Stop");
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });

        ControlButtonPanel.add(stopButton);

        resetButton.setText("Reset");
        resetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetButtonActionPerformed(evt);
            }
        });

        ControlButtonPanel.add(resetButton);

        newRecipieButton.setText("New Recipie");
        newRecipieButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        newRecipieButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newRecipieButtonActionPerformed(evt);
            }
        });

        ControlButtonPanel.add(newRecipieButton);

        getContentPane().add(ControlButtonPanel, java.awt.BorderLayout.SOUTH);

        fileMenu.setText("File");
        fileMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileMenuActionPerformed(evt);
            }
        });

        newRecipie.setText("New Recipie");
        newRecipie.setToolTipText("Create a new Etch-a-Sketch Recipie");
        newRecipie.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newRecipieActionPerformed(evt);
            }
        });

        fileMenu.add(newRecipie);
        advOptionsMenuItem.setText("Advanced Options...");
        advOptionsMenuItem.setToolTipText("Configure Advanced Etch-a-Sketch Options");
        advOptionsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                advOptionsMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(advOptionsMenuItem);
        fileMenu.add(jSeparator1);
        exitMenuItem.setText("Exit");
        exitMenuItem.setToolTipText("Exit the program");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);
        menuBar.add(fileMenu);
        recipieMenu.setText("Recipie");
        recipieMenu.addMenuListener(new javax.swing.event.MenuListener() {
            public void menuSelected(javax.swing.event.MenuEvent evt) {
                recipieMenuMenuSelected(evt);
            }
            public void menuDeselected(javax.swing.event.MenuEvent evt) {
            }
            public void menuCanceled(javax.swing.event.MenuEvent evt) {
            }
        });

        useVertCheckBoxMenuItem.setText("Use for Vertical");
        useVertCheckBoxMenuItem.setToolTipText("Use this rule to determine vertical position of the pen in the drawing window");
        useVertCheckBoxMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                useVertCheckBoxMenuItemActionPerformed(evt);
            }
        });

        recipieMenu.add(useVertCheckBoxMenuItem);
        useHorzCheckBoxMenuItem.setText("Use for Horizontal");
        useHorzCheckBoxMenuItem.setToolTipText("Use this rule to determine horizontal position of the pen in the drawing window");
        useHorzCheckBoxMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                useHorzCheckBoxMenuItemActionPerformed(evt);
            }
        });

        recipieMenu.add(useHorzCheckBoxMenuItem);
        recipieMenu.add(jSeparator2);
        compileRecipieMenuItem.setText("Compile");
        compileRecipieMenuItem.setToolTipText("Compile and load the code in this recipie");
        compileRecipieMenuItem.setEnabled(false);
        compileRecipieMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                compileRecipieMenuItemActionPerformed(evt);
            }
        });

        recipieMenu.add(compileRecipieMenuItem);
        compileBothRecipieMenuItem.setText("Compile Both");
        compileBothRecipieMenuItem.setToolTipText("Compile both vert and horz recipies");
        compileBothRecipieMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                compileBothRecipieMenuItemActionPerformed(evt);
            }
        });

        recipieMenu.add(compileBothRecipieMenuItem);
        standaloneRecipieMenuItem.setText("Standalone View");
        standaloneRecipieMenuItem.setToolTipText("View this recipie in it's own window");
        standaloneRecipieMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                standaloneRecipieMenuItemActionPerformed(evt);
            }
        });

        recipieMenu.add(standaloneRecipieMenuItem);
        menuBar.add(recipieMenu);
        setJMenuBar(menuBar);

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(400, 500));
        setLocation((screenSize.width-400)/2,(screenSize.height-500)/2);
    }//GEN-END:initComponents

    private void compileBothRecipieMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_compileBothRecipieMenuItemActionPerformed
        // Add your handling code here:
        synchronized (vert) {
            vMerger = new CodeMerger("Vertical",vert,yHandler,xHandler);
            vMerger.actionPerformed(evt);
            vert.setCompiled(true);
        }
        synchronized (horz) {
            hMerger = new CodeMerger("Horizontal",horz,xHandler,yHandler);
            hMerger.actionPerformed(evt);
            horz.setCompiled(true);
        }
    }//GEN-LAST:event_compileBothRecipieMenuItemActionPerformed

    private void advOptionsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_advOptionsMenuItemActionPerformed
        // Add your handling code here:
        myFrame.showAdvEnv();
    }//GEN-LAST:event_advOptionsMenuItemActionPerformed

    private void resetButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetButtonActionPerformed
        // Add your handling code here:
        myFrame.showResetMenu();
    }//GEN-LAST:event_resetButtonActionPerformed

    private void startButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startButtonActionPerformed
        // Add your handling code here:
        running = true;
        if(xStarted) {
            xSpirit.resume();
        } else {
            xSpirit.start();
            xStarted = true;
        }
        if(yStarted) {
            ySpirit.resume();
        } else {
            ySpirit.start();
            yStarted = true;            
        }
        myFrame.beginPainting();

    }//GEN-LAST:event_startButtonActionPerformed

    private void standaloneRecipieMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_standaloneRecipieMenuItemActionPerformed
        // Add your handling code here:
        new SoloRecipie(((RecipieView)recipieTabs.getSelectedComponent()).
            getRecipie(), this, recipieTabs.getTitleAt(
            recipieTabs.getSelectedIndex())).show();
        updateListeners();
    }//GEN-LAST:event_standaloneRecipieMenuItemActionPerformed

    private void compileRecipieMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_compileRecipieMenuItemActionPerformed
        // Add your handling code here:
        Recipie currentRecipie = ((RecipieView)recipieTabs.
            getSelectedComponent()).getRecipie();
        synchronized (currentRecipie) {
            if (isHorzRecipie(currentRecipie)) {
                hMerger = new CodeMerger("Horizontal",
                                         currentRecipie,xHandler,yHandler);
                hMerger.actionPerformed(evt);
            }
            if (isVertRecipie(currentRecipie)) {
                vMerger = new CodeMerger("Vertical",
                                         currentRecipie,yHandler,xHandler);
                vMerger.actionPerformed(evt);
            }
            ((RecipieView) recipieTabs.getSelectedComponent()).
                getRecipie().setCompiled(true);
        }
        compileRecipieMenuItem.setEnabled(false);
    }//GEN-LAST:event_compileRecipieMenuItemActionPerformed

    private void recipieMenuMenuSelected(javax.swing.event.MenuEvent evt) {//GEN-FIRST:event_recipieMenuMenuSelected
        // Add your handling code here:
        
        // Turn on the compile menuItem if the selected recipie has been 
        // compiled.
        boolean needsCompiled = !((RecipieView) recipieTabs.
            getSelectedComponent()).getRecipie().isCompiled();
        compileRecipieMenuItem.setEnabled(needsCompiled && !running);
        
        // Turn on the compile both menuitem if either the vert or the horz
        // recipies have not been compiled.
        boolean eitherNeedCompile = !(vert.isCompiled() && horz.isCompiled());
        compileBothRecipieMenuItem.setEnabled(eitherNeedCompile && !running);
    }//GEN-LAST:event_recipieMenuMenuSelected

    private void newRecipieButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newRecipieButtonActionPerformed
        // Add your handling code here:
        newRecipieActionPerformed(evt);
    }//GEN-LAST:event_newRecipieButtonActionPerformed

    private void newRecipieActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newRecipieActionPerformed
        // Add your handling code here:
        Recipie newRecipie = new Recipie();
        RecipieView newView = new RecipieView();
        newView.setRecipie(newRecipie);
        addEtchControlListener(newView);
        recipieTabs.add(newView);
                // Set the title on the tab to match the recipie's id num
        ((JTabbedPane) newView.getParent()).setTitleAt(
         ((JTabbedPane) newView.getParent()).indexOfComponent(newView), 
         "Recipie " + newView.getRecipieNum());
        updateListeners();
    }//GEN-LAST:event_newRecipieActionPerformed

    private void useHorzCheckBoxMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_useHorzCheckBoxMenuItemActionPerformed
        // Add your handling code here:
        if (useHorzCheckBoxMenuItem.getState() == true) {
            horz = ((RecipieView)recipieTabs.getSelectedComponent()).
                getRecipie();
        } else {
            horz = Recipie.nullRecipie;
        }
        updateListeners();
    }//GEN-LAST:event_useHorzCheckBoxMenuItemActionPerformed

    private void useVertCheckBoxMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_useVertCheckBoxMenuItemActionPerformed
        // Add your handling code here:
        if (useVertCheckBoxMenuItem.getState() == true) {
            vert = ((RecipieView)recipieTabs.getSelectedComponent()).
                getRecipie();
        } else {
            vert = Recipie.nullRecipie;
        }
        updateListeners();        
    }//GEN-LAST:event_useVertCheckBoxMenuItemActionPerformed

    private void recipieTabsStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_recipieTabsStateChanged
        // Add your handling code here:
        if (!ready2Go) { return; }  // this keeps us from processing events 
                                    // before the components in the tabs are 
                                    // created and added.
        
        useHorzCheckBoxMenuItem.setState(isHorzRecipie(
            ((RecipieView)recipieTabs.getSelectedComponent()).getRecipie())); 
        useVertCheckBoxMenuItem.setState(isVertRecipie(
            ((RecipieView)recipieTabs.getSelectedComponent()).getRecipie())); 
        //useHorzCheckBoxMenuItem
        updateListeners();
    }//GEN-LAST:event_recipieTabsStateChanged

    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        // Add your handling code here:
        running = false;
        this.xSpirit.suspend();
        this.ySpirit.suspend();
        myFrame.suspendPainting(); 
    }//GEN-LAST:event_stopButtonActionPerformed

    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        // Add your handling code here:
        exitForm(new java.awt.event.WindowEvent(this,
                                                WindowEvent.WINDOW_CLOSING));
    }//GEN-LAST:event_exitMenuItemActionPerformed

    private void fileMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileMenuActionPerformed
        // Add your handling code here:
    }//GEN-LAST:event_fileMenuActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        
        myFrame.setVisible(false);
        myFrame.dispose();
        xSpirit.stop();
        ySpirit.stop();
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    /**
     * Test a recipie to see if it is currently being used for the vertical axis.
     * Note that isHorzRecipie and isVertRecipie may both return true at the
     * same time.
     *
     * @param r The recipie to test
     * @return <code>true</code> if r is the vertical recipie, <code>false</code>
     */    
    public boolean isVertRecipie(Recipie r) {
        return r.getIdNum() == vert.getIdNum();
    }

    /**
     * Test a recipie to see if it is currently being used for the Horizontal axis.
     * Note that isHorzRecipie and isVertRecipie may both return true at the
     * same time.
     *
     * @param r The recipie to test
     * @return <code>true</code> if r is the horizontal recipie, <code>false</code>
     */    
    public boolean isHorzRecipie(Recipie r) {
        return r.getIdNum() == horz.getIdNum();
    }

    /** Regester a class that implements {@link EtchControlListener}. Listeners will
     * be notified when Recipies are added or removed, or assigned to the vertical
     * or horizontal rule positions.
     * @param el The listnener to add.
     */    
    public void addEtchControlListener(EtchControlListener el) {
        listeners.add(el);        
    }
    
    /** Unregister an EtchControlListener.
     * @param el The listner to unregister
     */    
    public void removeEtchControlListener(EtchControlListener el) {
        listeners.remove(el);
    }

    private void updateListeners() {
        for (int i=0; i<listeners.size(); i++){
            ((EtchControlListener) listeners.get(i)).etchControlUpdated(ce);
        }
    }

    
    /** The main point of entry for Spirograph when using this interface.
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        switch (args.length) {
            case 0: xArg = null; yArg = null; break; 
            case 1: xArg = args[0]; yArg = null; break; 
            case 2: xArg = args[0]; yArg =  args[1]; break; 
            default: System.out.println("Only  0, 1, or 2 arguments accpeted.");
        }
        
        if ( xArg == null ) {
            xAccel = new Dummy();
        } else {
            System.out.println ("Trying to get class: " + xArg);	
            xAccel = SpiroUtils.createAccel(xArg);	
        }
        
        if ( yArg == null )
            yAccel = new Dummy();
        else {
            System.out.println ("Trying to get class: " + yArg);	
            yAccel = SpiroUtils.createAccel(yArg);
        }
        
        if ((xAccel == null)||(yAccel == null)) {
            System.exit(0);
        }
        
        
        xHandler = new AccelHandler(xAccel, xCoord, yCoord);
        yHandler = new AccelHandler(yAccel, yCoord, xCoord);
        xSpirit = new Thread(xHandler);
        ySpirit = new Thread(yHandler);

        new EtchControl().show();
    }
    

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel titleLabel;
    private javax.swing.JPanel titleLabelPanel;
    private javax.swing.JSeparator jSeparator2;
    private spirograph.RecipieView recipieViewVert;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JCheckBoxMenuItem useHorzCheckBoxMenuItem;
    private javax.swing.JPanel ControlButtonPanel;
    private javax.swing.JButton stopButton;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JPanel centerPanel;
    private javax.swing.JCheckBoxMenuItem useVertCheckBoxMenuItem;
    private spirograph.RecipieView recipieViewHorz;
    private javax.swing.JMenuItem newRecipie;
    private javax.swing.JMenuItem advOptionsMenuItem;
    private javax.swing.JButton startButton;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JButton newRecipieButton;
    private javax.swing.JButton resetButton;
    private javax.swing.JMenu recipieMenu;
    private javax.swing.JTabbedPane recipieTabs;
    private javax.swing.JMenuItem compileRecipieMenuItem;
    private javax.swing.JMenuItem standaloneRecipieMenuItem;
    private javax.swing.JMenuItem compileBothRecipieMenuItem;
    // End of variables declaration//GEN-END:variables

    /*
     * $Log: EtchControl.java,v $
     * Revision 1.7  2003/01/15 18:26:50  gus
     * accomodate the renamining of editRecipie to setRecipie in RecipieVeiw
     *
     * Revision 1.6  2003/01/15 17:27:07  gus
     * Javadoc tweaks
     *
     * Revision 1.5  2003/01/14 22:50:04  gus
     * Added javadocs
     *
     * Revision 1.4  2003/01/08 18:51:02  gus
     * Added an item to the recipie menu that allows the user to compile both
     * rules at once in response to bug 39. Also shortend some long lines
     *
     * Revision 1.3  2003/01/06 18:45:59  gus
     * Prevent recompilation while code is running.
     *
     * Revision 1.2  2002/12/20 21:49:08  gus
     * Connect the new GUI with the old backend.
     *
     * Revision 1.1  2002/12/19 18:55:51  gus
     * A new GUI for editing and displaying the code to control the behavior of the ball.
     *
     */
}
