/*
 * Spirograph.java
 * part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.io.*;
import java.awt.*;

/** This class handles a lot of the initialization for the Spirograph.
 * It creates an instance of DotFrame and CodeBox and creates
 * instances of the name passed in on the command line.
 * It also creates the CodeBox and the AccelHandlers and passes
 * them to the appropriate objects. <p>
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering</p>
 *
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @version $Id: Spirograph.java,v 1.5 2004/02/09 20:55:03 gus Exp $
 * @see AccelHandler
 * @see CodeBox
 * @see DotFrame
 */
public class Spirograph {
    /** The initial height of the {@link DotPanel}.
     */    
  public static final int HEIGHT = 400;
  /** The initial width of the {@link DotPanel}.
   */  
  public static final int WIDTH = 400;
  /** Size of the dot as it will appear in the GUI. */
  public static final int BALLSIZE = 10;
  /** Minimum timestep for updates. */
  public static final double TIMESTEP = 1;    
  /** The strength of a gravity source.
   */  
  public static final int GRAV = 1000;
  /** Maximum dot velocity allowed. */
  public static final double MAXVEL = 60;
  /** The size of an elipse foci point.
   */  
  public static final int FOCUSSIZE = 6;

  /** Problem set wide default font. */
  public static final Font DEFAULTFONT = new Font("Plain", Font.PLAIN, 12);
    
  private Accelerator xAccel, yAccel;
  private Coord xCoord = new Coord();
  private Coord yCoord = new Coord();
  private AccelHandler xHandler, yHandler;
  private DotFrame myFrame;
  private CodeBox cb;
    
  /** 
   * Attempt to construct a new Spirograph with the given Accleration
   * Handlers.  Abort if either handler cannot be created.
   *
   * @param xArg The name of the file containing new AccelHandler for the x direction
   * @param yArg The name of the file containing new AccelHandler for the y direction
   */
  public Spirograph(String xArg, String yArg) {
	
    if ( xArg == null ) {
      xAccel = new Dummy();
    } else {
      System.out.println ("Trying to get class: " + xArg);	
      xAccel = SpiroUtils.createAccel(xArg);	
    }
	
    if ( yArg == null )
      yAccel = new Dummy();
    else {
      System.out.println ("Trying to get class: " + yArg);	
      yAccel = SpiroUtils.createAccel(yArg);
    }

    if ((xAccel == null)||(yAccel == null)) {
      System.exit(0);
    }
	
        
    xHandler = new AccelHandler(xAccel, xCoord, yCoord);
    yHandler = new AccelHandler(yAccel, yCoord, xCoord);

    cb = new CodeBox(xHandler, yHandler);
    
    myFrame = new DotFrame(xHandler, yHandler, xCoord, yCoord);
  }
}

/*
 * $Log: Spirograph.java,v $
 * Revision 1.5  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.4  2003/01/17 21:01:31  gus
 * Finish javadoc
 *
 * Revision 1.3  2003/01/15 17:36:10  gus
 * adding log keywords to files that don't have them
 *
 */